/// Parse an ACM CCS xml definition and generate a section for displaying it in the document
///
/// - acm-xml (string, raw): the ACM CCS XML
///
/// -> content
#let apply-acm-ccs(acm-xml) = {
  if type(acm-xml) == content {
    assert(acm-xml.func() == raw)
    acm-xml = acm-xml.text
  }
  let parsed = xml(bytes(acm-xml))
  assert(parsed.len() == 1)

  // Extract the version year?
  let year = parsed.at(0).tag.trim("ccs")

  // Collect concepts in dictionary, keyed by top level concept
  let concepts = (:)
  for concept in parsed.at(0).children.filter(c => type(c) == dictionary) {
    assert(concept.tag == "concept")

    let children = concept
      .children
      .filter(c => type(c) == dictionary)
      .map(c => {
        assert(c.children.len() == 1)
        (c.tag, c.children.at(0))
      })
      .to-dict()
    let (top-level, lower-level) = children.concept_desc.split("~")

    let lower-concepts = concepts.at(top-level, default: ())
    lower-concepts.push((
      description: lower-level,
      significance: int(children.concept_significance),
    ))
    concepts.insert(top-level, lower-concepts)
  }

  metadata(acm-xml) // This does nothing FIXME: actually embed metadata

  heading[Classification (ACM CCS #year)]

  list(
    ..concepts
      .pairs()
      .map(((top, lower)) => [
        #text(weight: "bold", top)
        #sym.arrow.r
        #(
          lower
            .map(c => {
              let weight = if c.significance == 500 {
                "bold"
              } else {
                "regular"
              }
              let style = if c.significance == 300 {
                "italic"
              } else {
                "normal"
              }
              text(weight: weight, style: style, c.description)
              ";"
            })
            .join(" ")
        )
      ]),
  )
}
