/// A state that keeps track whether the outline is being rendered
///
/// -> state
#let in-outline-state = state(
  "in-outline",
  false,
)

/// Render the custom outline
///
/// - style (dictionary): The style (see `style.typ`)
///
/// -> content
#let custom-outline(style) = {
  // Keep track of number sizes for alignment
  let outline-widths = state(
    "outline-width-measurements",
    (
      level-widths: (),
      page-width: 0cm,
    ),
  )
  // The dots used to fill empty space
  let dots = align(
    right,
    {
      repeat(
        gap: 0.5em,
        justify: false,
        text(..style.normal-text-args, weight: "regular")[.],
      )
    },
  )

  set outline.entry(fill: dots)
  show outline.entry: it => {
    set text(..style.heading-text-args, size: 0.9em) if it.level == 1
    set block(above: 1.75em) if it.level == 1

    let indent = if outline.indent == auto {
      (it.level - 1) * 1.6em
    } else if type(outline.indent) == length {
      (it.level - 1) * length
    } else {
      (outline.indent)(it.level - 1)
    }

    pad(
      left: indent,
      {
        it.prefix()
        h(1em)
        it.body()
        box(width: 1fr, it.fill)
        box(width: 2.5em, align(right, it.page()))
      }
    )
  }

  // Generate the outline (only outline headings at level 1 or 2)
  show heading: set block(spacing: 2em)
  in-outline-state.update(true)
  outline(depth: 2)
  in-outline-state.update(false)
  v(-1em)
}
