/// The default base style parameters used throughout the template.
///
/// It contains the following entries:
///
/// TODO: explain entries
///
/// -> dictionary
#let _DEFAULT_STYLE_BASE = (
  // Colors
  accent-color: red,
  code-background: white.darken(8%),
  // Text
  heading-text-args: arguments(
    font: "Fira Sans",
    weight: "semibold",
    size: 11pt,
  ),
  normal-text-args: arguments(
    font: "XCharter",
    number-type: "old-style",
    size: 11pt,
  ),
  raw-text-args: arguments(font: "Fira Sans"),
  // Misc
  title-image: image("../assets/FrontpageImage.svg"),
  // Todo default arguments
  todo: (
    enable: true,
    margin-inset: (inner: 1.5em, outer: 3.25em),
    color: red.lighten(60%),
    line-width: 0.75pt,
    underline-offset: 2pt,
    rect: rect.with(width: 100%, stroke: 0.5pt, radius: 0.5em),
  ),
)

/// The default document type presets, a collection of functions to change a style base,
/// depending on the document type which is being created.
///
/// -> dictionary
#let _DEFAULT_DOC_TYPE_PRESETS = (
  seminar: style => {
    (
      style
        + (
          accent-color: color.rgb("#57377C"),
          document-description: "Seminar Report",
        )
    )
  },
)

/// Global style state used to set style throughout the template
///
/// -> state
#let global-style = state("plt-template-style-state", none)

/// Generates the style parameters used throughout the template, expanding on a style base.
///
/// - document-type (string):
///
///   The document type for which to create the style, one of ["seminar"]
///   (see _DEFAULT_DOC_TYPE_PRESETS)
///
/// - style-base (dictionary):
///
///   The base for the style which will be generated
///   (see _DEFAULT_STYLE_BASE)
///
/// -> dictionary
#let default-style(
  document-type,
  style-base: _DEFAULT_STYLE_BASE,
  doc-type-presets: _DEFAULT_DOC_TYPE_PRESETS,
) = {
  assert(
    document-type in doc-type-presets,
    message: "Style must be one of: "
      + _DEFAULT_DOC_TYPE_PRESETS.keys().join(" | "),
  )

  (_DEFAULT_DOC_TYPE_PRESETS.at(document-type))(style-base)
}
