// Import relevant functions from the template
#import "/lib.typ": (
  template,
  todo,
  todo-missing-figure,
  utils.LaTeX,
  global-style,
)

#import "@preview/cetz:0.3.3"
#import "@preview/cetz-plot:0.1.1"

#show: template.with(
  document-type: "seminar",
  title: "Theses and Seminar Reports in the Programming Languages and Tools Group",
  subtitle: "A Template and Guide",
  running-title: "Reports and Theses in the PLT group",
  authors: (
    (
      name: "Christoph Bockisch",
      running-name: "C. Bockisch",
    ),
    (
      name: "Stefan Schulz",
      running-name: "S. Schulz",
    ),
    (
      name: "Teresa Dreyer",
      running-name: "T. Dreyer",
    ),
  ),
  supervisors: (
    (
      role: "Supervisor",
      info: [
        Prof. Dr.-Ing. C. Bockisch

        _Programming Languages \ and Tools Group_
      ],
    ),
    (
      role: "Advisor",
      info: [
        M.Sc. Teresa Dreyer

        _Programming Languages \ and Tools Group_
      ],
    ),
  ),
  abstract: [
    The summary should provide a cross-section of the entire work, i.e. from the motivation to
    the conceptual approach, the execution and evaluation through to the result of the work.
    Only the most important points and conclusions should be briefly mentioned. For theses,
    an additional German Zusammenfassung is needed if the thesis is in English.
  ],
  appendix: [
    = A Famous Filler Text

    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor
    incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis
    nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat.
    Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu
    fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in
    culpa qui officia deserunt mollit anim id est laborum.

    = Glossar

    If it makes sense in your case, you can include a glossary.
    Please be aware that the #LaTeX() package `glossary` is not compatible with the document class and can not be used or included.
  ],
  bibliography-path: "/template/example.bib",
  acm-ccs: ```
    <ccs2012>
    <concept>
    <concept_id>10010405.10010489</concept_id>
    <concept_desc>Applied computing~Education</concept_desc>
    <concept_significance>500</concept_significance>
    </concept>
    <concept>
    <concept_id>10010405.10010497.10010510</concept_id>
    <concept_desc>Applied computing~Document preparation</concept_desc>
    <concept_significance>500</concept_significance>
    </concept>
    <concept>
    <concept_id>10011007.10011006</concept_id>
    <concept_desc>Software and its engineering~Software notations and tools</concept_desc>
    <concept_significance>300</concept_significance>
    </concept>
    </ccs2012>
  ```,
  keywords: (
    "thesis",
    "seminar paper",
    "format template",
    "structure template",
    "good design",
  ),
)

#set text(lang: "en")

#let todo-cb = todo.with(color: orange.lighten(70%), author: "CB")

= General information on Seminar Papers

The length of the written paper should be approximately 3000 words per person.
This corresponds to about 6 pages.
This does not include the cover sheet, bibliography and appendix.
You can choose whether you want to write the paper in German or English.

If you have programmed as part of your seminar paper, please provide us with the code via git.
Use the university gitlab for this.
The same applies for your data and the complete script with your statistical analysis, if applicable.
If you are dealing with sensitive data, please contact us. Do not upload them anywhere unless we have explicitly okayed this.
You do not have to put the entire code in the appendix of the written paper, but can refer to the repository.
The same applies for statistical analyses.
Individual code examples in the body text should still be used where they make sense.

= Structure of the Report: Seminar Empirical Software Engineering

The structure of the written paper generally depend on the actual seminar.
This section is specific for the seminar "Empirical Software Engineering".
For hints on the structure for other seminars in the research group, in particular "Debugging, Testing and Improving Programs", refer to the following section.

Your written paper should contain a short introduction and a short conclusion, if appropriate with an outlook.
In the main part of your paper, you should explain the statistical analysis, briefly present the selected paper and critically examine it with regard to the statistical analysis.
Also explain the conditions under which the method can/may be used and analyze whether the conditions for your paper are met.
Please also refer to the slides from the kick-off meeting and any special features discussed individually that are relevant to your topic.

The presentation of the procedure should take up about half or less space. The critical analysis of the paper should be in the foreground, the summary of the paper should be kept as short as possible.

We do not prescribe the exact structure of the paper. For example, the structure could look like this

#block(breakable: false)[
  - Introduction
  - Description of the Statistical Method
  - Brief Summary of the Selected Paper
  - Critical Analysis of the Selected Paper
  - Conclusion
]

// Hacky, since typst does not continue first line indents after blocks other than paragraphs
Attention: You need to adapt the template to reflect your topic and advisor.

= Structure of the Report: Seminar Debugging, Testing and Improving Programs

Your written paper should contain a short introduction and a short conclusion, if appropriate with an outlook.

In the main part of your paper, you should summarize the seed paper that was provided to you and summarize your chosen additional paper. As part of this seminar, you are supposed to work out an example centered around a software error scenario to demonstrate the approach of one of the papers or both papers. This should also be discussed in the main part of the paper and should make up a substantial part of your report. Minimally, this should be the example used by the paper authors. Better would be your own example, if possible use the same example to demonstrate the approaches of both papers. Ideally you should practically use the tool or approach presented in the paper(s) and reproduce the evaluation. Your choices here will influence your grade, please also see the slides from the kick-off meeting. When in doubt, please contact your advisor.

Attention: You need to adapt the template to reflect your topic and advisor.

= Literature

The majority of the sources used should be scientific, peer-reviewed sources.
This means publications in conference proceedings or journals.
Workshop publications typically have a lower status, but are still peer-reviewed.

Books are also often trustworthy sources and fully citable.
However, there is not always a reviewing body for books, such as the peer review process for conferences, journals and workshops.
Other sources, such as white papers, weblogs, documentation, etc. can also be used and referenced.
For these sources in particular---but also for all sources you use in general---you must check for yourself whether you consider them trustworthy. #footnote[see #link("http://www.findingdulcinea.com/news/education/2010/march/The-Top-10-Reasons-Students-Cannot-Cite-or-Rely-on-Wikipedia.html").]
For example, you can distinguish between genuine publications (scientific articles and books) and other sources by including only the genuine publications in your bibliography.
You can list the other sources as footnotes (see above).
This also makes sense because these "other sources" are often only used in one place in the document. For specific information on reference management with LaTeX, see section @sec:bibliography. // TODO: add verweis zu literaturverzeichnis

= Practical Tips for Using #LaTeX(font: "Fira Sans", weight: "semibold")

This section contains some practical tips for using #LaTeX() to structure your document and influence the appearance.
It also presents some useful packages that can be used and points out common pitfalls.

== Hyphenation

LaTeX automatically performs hyphenation using the rules for the language that you have specified as an option for the document class (see comments in the sample text file).
If a word is not hyphenated correctly, you can define your own hyphenation for this word.
A separate rule must always be defined for this.
You can define new rules using the command `hyphenation{}` that apply throughout the rest of the document.

```
\hyphenation{su-per-cal-i-frag-i-lis-tic-ex-pi-al-i-do-cious}

I think this is: supercalifragilisticexpialidocious
```

#context rect(width: 2.2in, fill: global-style.get().code-background)[
  I think this is: supercalifragilisticexpialidocious
]
#v(1em)

LaTeX can separate compound words with a hyphen automatically, except at the position of the hyphen:

#context rect(width: 2.2in, fill: global-style.get().code-background)[
  #set text(hyphenate: false)
  Einnahmen-Überschuss-Rechnung
]
#v(1em)

Instead of the hyphen, use the command `\hyp{}` from the package `hyphenat` to also apply hyphenation to the partial words:

```
Einnahmen\hyp{}Überschuss\hyp{}Rechnung
```

#context rect(width: 2.2in, fill: global-style.get().code-background)[
  Einnahmen-Überschuss-Rechnung
]

== Quoation marks and dashes

If you want to use quotation marks, you must not simply use the corresponding ASCII symbol in LaTeX, as this has a control function.
This document class offers the command `\quot{}` to set text in quotation marks.
The quotation marks depend on the language setting that was passed to the class.
The specified text itself can also use LaTeX commands.
For example: `\quot{Auch der \textbf{Tüchtige} braucht \textit{Glück}.}` results in: "Auch der *Tüchtige* braucht _Glück_."

LaTeX recognizes different forms of dashes: `-` (-) "hyphen or in German Bindestrich-Minus", `--` (--) "en-dash or in German Halbgeviertstrich", `---` (---) "em-dash, in German Geviertstrich", each with different tasks.
You can find a discussion of how to use these forms of strokes here:

- in English texts: #link("https://www.thepunctuationguide.com/index.html")
- in German texts:
  - #link("https://de.wikipedia.org/wiki/Geviertstrich")
  - #link("https://de.wikipedia.org/wiki/Halbgeviertstrich")
  - #link("https://de.wikipedia.org/wiki/Viertelgeviertstrich#Bindestrich-Minus")

== TODOs

To insert TODOs or comments into the document, you can use the `todonotes` package, which is already loaded.
See also #link("http://tug.ctan.org/macros/latex/contrib/todonotes/todonotes.pdf").
A list of all TODOs is automatically displayed at the beginning of the document so that you have an overview of whether you have processed all of them.

You can switch off this display of TODOs in the document by passing the option `notodo` to the document class in your Latex file: `\documentclass[ngerman,bachelor,notodo]{pltdoc}`
Then the overview at the beginning of the document is no longer displayed.
You should use this option when you create the final version of your thesis.

You can insert a TODO comment using the command `\todo{Find a good example}`#todo[Find a good example].
Parameters can optionally be defined for the command in square brackets.
For example, long comments are best inserted as inline comments, for which the `inline` parameter is specified: `\todo[inline]{This comment is much too long for the correction margin and would not be displayed legibly}` #todo(inline: true)[This comment is much too long for the correction margin and would not be displayed legibly].

You can use other parameters to define the color or the author of the comment.
You can also define a short form of the comment that is displayed in the overview.
Consult the package documentation for this.

To indicate that an image is missing at a certain point, you can use the command `\missingfigure{Here comes another illustrative image}`
The size of the placeholder can be determined via parameters.

#todo-missing-figure("An illustrative image is added here")

You can also use the command in a float environment.
It is then treated like a normal figure (see @fig:missing) and can later be replaced by `\includegraphics{}`

```
\begin{figure}
\missingfigure{Here, a picture is missing.}
\caption{Missing Picture}
\label{fig:missing}
\end{figure}
```

#figure(
  placement: auto,
  caption: [Missing Picture],
  todo-missing-figure("Here another picture is missing."),
) <fig:missing>

The LaTex template defines a command for the creation of shortcuts for TODO macros.
Regarding this, have a look at the LaTeX comment over the `\begin{document}` command.

Examples:

#block(spacing: 1em)[
  #set list(marker: none)

  - `\cb{Test1}`
  - `\cb[Test2]{Test3}`
  - `\cbinline[Test2]{Test3}`
  - `\cbfigure{Test4}`
]

#todo-cb[Test1] #todo-cb[Test2][Test3] #todo-cb(inline: true)[Test2][Test3]
#todo-missing-figure(author: "CB")[Test4]

== Source Code Representation

The class is based on the template provided by the Programming Journal (see #link("https://github.com/programming-journal/programming/releases")). The Programming Journal Template is based on the KOMA-Script Article (scrartcl, siehe #link("https://komascript.de/komascriptbestandteile")).
This means that `\chapter` *not* is available.
The commands `\frontmatter`, `\mainmatter` and `\backmatter` cannot be used either.
The following structure levels are therefore available: `\section`, `\subsection`, `\subsubsection`, `\paragraph` and `\subparagraph`

Use the listings package to display source code (#link("https://ftp.agdsn.de/pub/mirrors/latex/dante/macros/latex/contrib/listings/listings.pdf")).
Minted is currently not correctly supported by this document class, for example the listings directory is not created correctly with Minted.

You can display source code via the environment `\begin{lstlisting} ... \end{lstlisting}` as a block (see @lst:listings).
You can either specify the language used in the source code as an optional parameter for each listing (`\begin{lstlisting}[language=Java] ...`) or use `\lstset{language=Java}` to set the default language from this command onwards (the language can be explicitly overwritten by specifying the optional parameter).
In addition to displaying as a code block, you can also display formatted source code in continuous text. To do this, use the command `\lstinline!if (dndResult == null) {}!` (results in: `if (dndResult == null) {}`).
The default language is also taken into account here and can also be specified explicitly using an optional parameter: `\lstinline[language=Java]!if (dndResult == null) {}!`.
If the source text that you want to display contains an exclamation mark (!), you can also choose a different delimiter, e.g. `\lstinline|!|` results in `!`.

Inline-Listings are not wrapped in an environment with justified text.
You should therefore avoid long inline listings.

#figure(
  caption: [Example for Java source code with the listings package.],
  [
    #set raw(tab-size: 8)
    ```java
    // Register a callback for the Mouse Up event
    canvas.addListener(SWT.MouseUp, e -> {
        synchronized (GUI.this) {
            if (dndHandler != null && cardBeingMoved != null) {
                Coordinate dndResult = dndHandler.cardDraggedAndDropped(cardBeingMoved,
                        new Coordinate(e.x, e.y), new Coordinate(e.x - deltaX, e.y - deltaY));
                if (dndResult != null) {
                    cardBeingMoved.x = dndResult.x;
                    cardBeingMoved.y = dndResult.y;
                }
            }
            cardBeingMoved = null;
            zoomedCard = null;
            mouseUpCounter += 1;
            canvas.redraw();
            Logger.global.log(Level.INFO, "Mouse Up event");
        }
    });
    ```
  ],
)<lst:listings>

You can use the German Umlauts ä, ö, ü, Ä, Ö, Ü und ß in Listings.
If you want to use other special characters that the listings package cannot normally handle, write the following lines at the beginning of your document and add to the list of special characters.
Add the following to "literate" for each special character:
In curly brackets the character that you want to use in the listings environment, in curly brackets the command to generate the character, the character "1".
The output command must be enclosed in curly brackets again.
The '1' means that the inserted latex command stands for a character in the listing.
For better clarity, you can separate the entries with spaces.
You can also replace longer character strings, e.g. if you want to use formula characters or Greek letters in pseudocode.
See also section 6.4 in #link("http://texdoc.net/texmf-dist/doc/latex/listings/listings.pdf").

#[
  #show "α": text(style: "italic", "α")
  ```latex
  \lstset{
      extendedchars=true,
      literate={ä}{{\"a}}1 {ö}{{\"o}}1 {ü}{{\"u}}1 {Ä}{{\"A}}1 {Ö}{{\"O}}1 {Ü}{{\"U}}1 {ß}{{\ss}}1 {\@alpha}{{$alpha{}$}}1,
  }

  if α then true else false
  ```
]

== Structure: Subsection

This is the beginning of a section.
The commands for structuring the text are:

+ `\part{heading}` (numbered) or `\part*{heading}` (not numbered)
+ `\section{heading}` (numbered) or `\section*{heading}` (not numbered)
+ `\subsection{heading}` (numbered) or `\subsection*{heading}` (not numbered)
+ `\subsubsection{heading}` (numbered) or `\subsubsection*{heading}` (not numbered)
+ `\paragraph{heading}` (numbered) or `\paragraph*{heading}` (not numbered)
+ `\subparagraph{heading}` (numbered) or `\subparagraph*{heading}` (not numbered)

If possible, do not use structures that are too deep.
Typically, the structuring levels section -- subsubsection are sufficient.
You can also divide the work into parts using the part structuring level.
Only use this if your work deals with several independent focal points and the parts each have a certain size that justifies further subdivision.

Also pay attention to the balance of the structure. Very short sections or subsections should be avoided.
Also, it is typically not good to have a single subsection in a section (this also applies to the other levels of structure).
Below you will find examples of the structuring levels.

== Subsection

This is a subsection.

=== Subsubsection

This is a subsubsection.

==== Paragraph

This is a paragraph.

===== Subparagraph

This is a sub paragraph.

== References

To refer to other text passages in the text, you must first mark the target passage with a label.
To do this, use the command `\label{thelable}`
A label can be defined directly after a structure command (e.g. `\section{...}`).
To mark graphics, tables, etc., define the label *behind* the `\caption{...}` command in the corresponding environment:

#figure(caption: [Listing with an example of a label.])[
  ```latex
  \begin{figure}
  \begin{center}
  \includegraphics[width=1.0\textwidth]{architecture.png}
  \end{center}
  \caption{Architektur-Diagramm}
  \label{fig:architecture}
  \end{figure}
  ```
]<lst:example-label>

You can define a label for a listing using an optional parameter.
You must then also define a caption for the listing.

```latex
\begin{lstlisting}[language=TeX,
        label={lst:example-label},
        caption={Listing with an example of a label.}]
\begin{figure}
% ...
\end{figure}
\end{lstlisting}
```

To refer to the text passage with the label, you can use the command `\ref{lst:example-label}`
This generates the number of the element with the label: @lst:example-label.
The command `\autoref{lst:example-label}` creates a reference consisting of the type of object being referenced and its number: @lst:example-label.
The name of the label is irrelevant.
The distinction between the object types in the label (e.g. "lst:" for listings) is merely a convention.

== Bibliography <sec:bibliography>

To manage the literature used, use Bibtex (see #link("http://www.bibtex.org/Using/de/")).
To do this, create an entry in a Bibtex file, e.g. "example.bin", for each source you use.
In the document, you can use `\cite{key}` to reference the source with the key "key", for example `\cite{Georges2007}` @Georges2007.
You can use the optional parameter (in square brackets) to specify additional information, such as the chapter within a book, for a citation: `\cite[Section 4.10]{Gosling:2014}` results in @Gosling:2014[Section~4.10].

The bibliography is inserted into the document using the following commands:

```
\cleardoublepage
\bibliography{example}
\cleardoublepage
```

You can often download the entries for the Bibtex file or have them generated.
For example, publishers such as ACM, Springer or IEEE offer the option of downloading the Bibtex entry for one of their articles.
You can also use tools such as Mendeley (see #link("https://www.mendeley.com")) to manage your Bibtex file.

For references, use the `\textbackslash cite` command, e.g. @Gude:2014, @Oderski:2009 @Schlosser:2017 or @Gosling:2014[section 4.10].

== Figures, Tables, Listings

Sometimes it is necessary to pay special attention to the _*floating elements*_ of your document, that is figures, tables, listings, and similar.
While it is tempting to force those elements to the specific position where
they are written down, this is not always the best choice, especially since it
can interrupt the reading flow. Therefore please refrain form using the
`[h!]` and `[H]` specifiers for such elements. If you
_*must*_ have the element at the current position, it maybe is not a
_*floating element*_, in the first place. It is perfectly fine to use
graphics _*without*_ a `\begin{figure}...\end{figure}` environment
or use a `\begin{tabular}...\end{tabular}` _*without*_ an
enclosing `\begin{table}...\end{table}`.

The caption position of a floating element should match its content. For
tables, listings, algorithms, or similar list-like content, please use captions
_*above*_ its contents, as those elements are typically read from top to
bottom. Find examples in the listings already presented and
@tbl:things. For figures, graphs, graphics, or similar, please us
captions _*below*_ the imagery, as in @fig:test.

#figure(
  caption: [A test figure],
  scale(
    125%,
    reflow: true,
    cetz.canvas(
      length: 0.5cm,
      {
        import cetz.draw: *

        let axis = line.with(
          stroke: 0.3pt,
          mark: (end: (symbol: ">", fill: black, scale: 0.5)),
        )

        axis((-1, 0), (7, 0))
        axis((0, -1), (0, 2))

        line((-0.15, 1), (0.15, 1), stroke: 0.3pt, name: "tick")
        content(
          "tick.start",
          padding: 1pt,
          anchor: "east",
          text(size: 0.8em, $1$),
        )

        line((2, -0.15), (2, 0.15), stroke: 0.3pt, name: "tick")
        content("tick.start", anchor: "north", text(size: 0.8em, $pi$))

        line((4, -0.15), (4, 0.15), stroke: 0.3pt, name: "tick")
        content("tick.start", anchor: "north", text(size: 0.8em, $2 pi$))

        set-style(stroke: 0.75pt)

        line((0, 0), (2, 0))
        line((2, 1), (4, 1))
        line((4, 0), (6.9, 0))

        content((5.5, 1), anchor: "south-west", text(size: 0.8em, $g(t)$))
        content((6.8, 0), anchor: "north", padding: 0.1, text(size: 0.8em, $t$))
      },
    ),
  ),
) <fig:test>

#figure(
  caption: [Differences between things projected and things achieved],
  text(
    size: 0.85em,
    table(
      stroke: none,
      columns: (4.5cm, auto, auto, auto),
      align: (left, auto, auto, auto),
      column-gutter: 1cm,
      row-gutter: 0pt,
      table.hline(),
      [Part], [done], [Value], [Unit],
      table.hline(),
      [Title], [yes], [2.3456], [dB],
      [Abstract], [yes], [10#super(typographic: false)[3]], [khz],
      table.hline(),
      table.cell(align: center, colspan: 4)[Rest is not entirely true],
      table.hline(),
      [Context], [yes], [90.473], [%],
      [Problem], [no#super[a]], [], [],
      [Solution], [yes], [5642.5], [MB],
      [Implementation], [yes], [1.2e-3], [m#super(typographic: false)[2]/s],
      [Evaluation], [no], [], [],
      [Related Work], [no], [], [],
      [Conclusion], [yes], [4955.3], [/kg],
      table.hline(),
      table.cell(colspan: 4, align(left)[#super[a] Just a few things missing]),
    ),
  ),
) <tbl:things>

For tabular content, we suggest to avoid vertical lines altogether and keep
horizontal lines to a minimum. Refer to the `booktabs` package
documentation for more information~@booktabs. Also, the
`threeparttable` package is valuable if you need footnotes in your
table~@threeparttable.


#todo-missing-figure(height: 8cm)[Missing class diagram]

== LaTeX Packages Considerations <sec:package-cons>

The following packages are automatically loaded by the \lstinline|programming|
class and need not to be loaded manually.

#{
  set text(fill: green.darken(40%))
  block(
    spacing: 2em,
    breakable: true,
    context grid(
      columns: (1fr,) * 4,
      gutter: list.spacing,
      [- accsupp], [- amsmath], [- amstext], [- array],
      [- atbegshi], [- babel], [- booktabs], [- calc],
      [- caption], [- colortbl], [- comment], [- csquotes],
      [- doclicense], [- expl3], [- environ], [- etoolbox],
      [- FiraSans], [- fnpct], [- fontspec], [- fontenc],
      [- gettitlestring], [- graphics], [- graphicx], [- grfext],
      [- ifthen], [- iflang], [- hypcap], [- hyphenat],
      [- hyperref], [- hyperxmp], [- listings], [- mathdesign],
      [- microtype], [- morewrites], [- multirow], [- multicol],
      [- relsize], [- rotating], [- siunitx], [- subcaption],
      [- tabularx], [- textcase], [- textcomp], [- tikz],
      [- tikz-uml], [- threeparttable], [- tocbibind], [- tocloft],
      [- typearea], [- url], [- verbatim], [- xcolor],
      [- xparse], [- xspace], [- xunicode],
    ),
  )
}

The following packages _*must not*_ be loaded.

#{
  set text(fill: red.darken(40%))

  context block(
    spacing: 2em,
    breakable: true,
    grid(
      columns: (1fr,) * 4,
      gutter: list.spacing,
      [- SIstyle], [- SIunits], [- a4wide], [- a4],
      [- aecompl], [- ae], [- caption2], [- courier],
      [- doublespace], [- epsfig], [- epsf], [- euler],
      [- fancyhdr], [- fancyheadings], [- fourier], [- geometry],
      [- glossary], [- helvet], [- isolatin], [- mathpple],
      [- mathptmx], [- mathptm], [- newtxmath], [- newtxtext],
      [- palatino], [- psfig], [- pslatex], [- scrpage],
      [- subfigure], [- subfig], [- t1enc], [- times],
      [- umlaut], [- utopia], [- zefonts],
    ),
  )
}

Additionally, any package that changes the font must not be loaded.
